import grok
import re
from zope.app.authentication.authentication import PluggableAuthentication
from zope.app.security.interfaces import IAuthentication
from zope.component import getUtility, getMultiAdapter, createObject
from zope.interface import Interface

from waeup.interfaces import (IUniversity, IStudentContainer, IHostelContainer,
                              IFacultyContainer)
from waeup.viewlets import MainArea, LeftSidebar, Index, Manage, FormWrapMixin
from waeup.authentication import setup_authentication
from waeup.users import UserContainer

class University(grok.Application, grok.Container):
    """A university.
    """
    grok.implements(IUniversity)
    # Setup authentication for this app. Note: this is only
    # initialized, when a new instance of this app is created.
    grok.local_utility(
        PluggableAuthentication, provides = IAuthentication,
        setup = setup_authentication)

    def __init__(self, name=u'Sample University', **kw):
        super(University, self).__init__(**kw)
        self.name = name
        self.setup()

    def setup(self):
        self.students = createObject(u'waeup.StudentContainer')
        self.hostels = createObject(u'waeup.HostelContainer')
        self.faculties = createObject(u'waeup.FacultyContainer')
        self.users = UserContainer()

    def traverse(self, name):
        # If someone requests a subobject 'faculties', we return our
        # `faculties` attribute.
        if name == 'faculties':
            return self.faculties
        elif name == 'hostels':
            return self.hostels
        elif name == 'students':
            return self.students
        elif name == 'users':
            return self.users
        return None
        
        
class ManageForm(grok.EditForm):
    """Manage the basic properties of a `University` instance.
    """

    form_fields = grok.AutoFields(IUniversity)
    grok.require('waeup.manageUniversity')

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context))
        return

class ManageUniversity(Manage):
    grok.name('manage')
    grok.context(IUniversity)
    grok.require('waeup.manageUniversity')

    def render(self, *args, **kw):
        return super(ManageUniversity, self).render(*args, **kw)
    
class ManageUniversityMain(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `ManageForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IUniversity)
    grok.view(Manage)
    grok.require('waeup.manageUniversity')
    
    formview_name = 'manageform' # The name of the formview we want to
                                 # be rendered in this viewlet.

class Content(grok.Viewlet):
    grok.viewletmanager(MainArea)
    grok.context(IUniversity)
    grok.view(Index)

class ManageLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(Index)
    grok.order(5)
    # This link is only displayed, if the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="manage">Settings</a></div>'

class ManageFacultiesLink(grok.Viewlet):
    grok.viewletmanager(LeftSidebar)
    grok.context(IUniversity)
    grok.view(Index)
    grok.order(6)
    # This is so cool! This link is only displayed, when the user is
    # allowed to use it!
    grok.require('waeup.manageUniversity')
    
    def render(self):
        return u'<div class="portlet"><a href="faculties">Manage faculties</a></div>'
